------------------------------------------------------------------------------
-- $Id: registers.vhd,v 1.3 2008/06/19 21:41:00 mwelter Exp $
------------------------------------------------------------------------------
-- registers - entity and arch
------------------------------------------------------------------------------
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        registers.vhd
-- Version:         v1.01a
-- Description:     Include a meaningful description of your file. Multi-line
--                  descriptions should align with each other
--
--  Addr   REG    Chipselect #
-- offset
-- 0x000   RAF0   Bus2IP_Wr/RdCE(0)
-- 0x004   TPF0   Bus2IP_Wr/RdCE(1)
-- 0x008   IFGP0  Bus2IP_Wr/RdCE(2)
-- 0x00C   IS0    Bus2IP_Wr/RdCE(3)
-- 0x010   IP0    Bus2IP_Wr/RdCE(4)
-- 0x014   IE0    Bus2IP_Wr/RdCE(5)
--
-- 0x020   MSW0   Bus2IP_Wr/RdCE(8)
-- 0x024   LSW0   Bus2IP_Wr/RdCE(9)
-- 0x028   CTL0   Bus2IP_Wr/RdCE(10)
-- 0x02C   RDY0   Bus2IP_Wr/RdCE(11)
--
-- 0x040   RAF1   Bus2IP_Wr/RdCE(16)
-- 0x044   TPF1   Bus2IP_Wr/RdCE(17)
-- 0x048   IFGP1  Bus2IP_Wr/RdCE(18)
-- 0x04C   IS1    Bus2IP_Wr/RdCE(19)
-- 0x050   IP1    Bus2IP_Wr/RdCE(20)
-- 0x054   IE1    Bus2IP_Wr/RdCE(21)
--
-- 0x060   MSW1   Bus2IP_Wr/RdCE(24)
-- 0x064   LSW1   Bus2IP_Wr/RdCE(25)
-- 0x068   CTL1   Bus2IP_Wr/RdCE(26)
-- 0x06C   RDY1   Bus2IP_Wr/RdCE(27)
--
-- C_EMAC0_DCRBASEADDR : bit_vector(9 downto 0) := "0000000000";
-- C_EMAC1_DCRBASEADDR : bit_vector(9 downto 0) := "0000000100";
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the 
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      
-- History:
--
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.numeric_bit.all;
use ieee.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.all;

library unisim;
use unisim.vcomponents.all;

library xps_ll_temac_v1_01_b;

entity registers is
  generic
  (
    C_TEMAC1_ENABLED : integer          := 1
  );
  port
  (
    DCR_Clk      : out std_logic;
    DCR_Read     : out std_logic;
    DCR_Write    : out std_logic;
    DCR_Ack      : in  std_logic;
    DCR_ABus     : out std_logic_vector(0 to 9);
    DCRTemac_DBus: out std_logic_vector(0 to 31);
    TemacDcr_DBus: in  std_logic_vector(0 to 31);
    CLK          : in  std_logic;
    RawReset     : in  std_logic;
    IP2Bus_Data  : out std_logic_vector(0 to 31);
    IP2Bus_WrAck : out std_logic;
    IP2Bus_RdAck : out std_logic;
    Bus2IP_Addr  : in  std_logic_vector(0 to 31);
    Bus2IP_Data  : in  std_logic_vector(0 to 31);
    Bus2IP_RNW   : in  std_logic;
    Bus2IP_CS    : in  std_logic;   
    Bus2IP_RdCE  : in  std_logic_vector(0 to 27);     
    Bus2IP_WrCE  : in  std_logic_vector(0 to 27);     
    Intrpts0     : in  std_logic_vector(24 to 31);
    Intrpts1     : in  std_logic_vector(24 to 31);
    TPReq0       : out std_logic;
    TPReq1       : out std_logic;
    Cr0RegData   : out std_logic_vector(29 to 31);
    Cr1RegData   : out std_logic_vector(29 to 31);
    Tp0RegData   : out std_logic_vector(16 to 31);
    Tp1RegData   : out std_logic_vector(16 to 31);
    Ifgp0RegData : out std_logic_vector(24 to 31);
    Ifgp1RegData : out std_logic_vector(24 to 31);
    Is0RegData   : out std_logic_vector(24 to 31);
    Is1RegData   : out std_logic_vector(24 to 31);
    Ip0RegData   : out std_logic_vector(24 to 31);
    Ip1RegData   : out std_logic_vector(24 to 31);
    Ie0RegData   : out std_logic_vector(24 to 31);
    Ie1RegData   : out std_logic_vector(24 to 31);
    Intrpt0      : out std_logic;
    Intrpt1      : out std_logic
  );
end registers;

------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------

architecture imp of registers is

------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------
signal dCR_Read_i     : std_logic;
signal dCR_Write_i    : std_logic;
signal dCR_ABus_i     : std_logic_vector(0 to 9);
signal dCRTemac_DBus_i: std_logic_vector(0 to 31);

signal cr1RdData     : std_logic_vector(29 to 31);
signal tp1RdData     : std_logic_vector(16 to 31);
signal ifgp1RdData   : std_logic_vector(24 to 31);
signal is1RdData      : std_logic_vector(24 to 31);
signal ie1RdData      : std_logic_vector(24 to 31);
signal ip1RdData      : std_logic_vector(24 to 31);
signal is1RegData_i   : std_logic_vector(24 to 31);
signal ie1RegData_i   : std_logic_vector(24 to 31);
signal rdData        : std_logic_vector(0 to 31);
--signal wrData        : std_logic_vector(0 to 31);
signal cr0RdData     : std_logic_vector(29 to 31);
signal tp0RdData     : std_logic_vector(16 to 31);
signal ifgp0RdData   : std_logic_vector(24 to 31);
signal is0RdData     : std_logic_vector(24 to 31);
signal ie0RdData     : std_logic_vector(24 to 31);
signal ip0RdData     : std_logic_vector(24 to 31);
signal is0RegData_i  : std_logic_vector(24 to 31);
signal ie0RegData_i  : std_logic_vector(24 to 31);
signal softRead0     : std_logic;
signal softRead1     : std_logic;
signal softWrite0    : std_logic;
signal softWrite1    : std_logic;
signal dCR_Read0     : std_logic;
signal dCR_Read1     : std_logic;
signal dCR_Write0    : std_logic;
signal dCR_Write1    : std_logic;
signal dcrTemac1Op   : std_logic;
signal temacDcr_DBus_i : std_logic_vector(0 to 31);
signal dCR_Ack_i     : std_logic;
signal softRead0_d1  : std_logic;
signal softRead1_d1  : std_logic;
signal softWrite0_d1 : std_logic;
signal softWrite1_d1 : std_logic;
signal dCR_Read0_d1  : std_logic;   
signal dCR_Write0_d1 : std_logic;   
signal dCR_Read1_d1  : std_logic;   
signal dCR_Write1_d1 : std_logic; 

signal iP2Bus_WrAck_i: std_logic;
signal iP2Bus_RdAck_i: std_logic;

signal rdAckBlocker  : std_logic;
signal wrAckBlocker  : std_logic;

begin
  
  IP2Bus_WrAck<= iP2Bus_WrAck_i;
  IP2Bus_RdAck<= iP2Bus_RdAck_i;
  
  pipel_PROCESS : process (RawReset, CLK)
  begin
    if (RawReset = '1') then
      DCR_Read      <= '0';
      DCR_Write     <= '0';
      DCR_ABus      <= (others => '0');
      DCRTemac_DBus <= (others => '0');
    elsif (CLK'event and CLK = '1') then
      DCR_Read      <= dCR_Read_i;     
      DCR_Write     <= dCR_Write_i;    
      DCR_ABus      <= dCR_ABus_i;     
      DCRTemac_DBus <= dCRTemac_DBus_i;
    end if;
  end process;

  Is0RegData  <= is0RegData_i;
  Ie0RegData  <= ie0RegData_i;

  CR0_I :  entity xps_ll_temac_v1_01_b.reg_cr(imp)
    port map
    (
     Clk      => CLK,                   -- in
     RST      => RawReset,              -- in
     RawReset => RawReset,              -- in
     RdCE     => Bus2IP_RdCE(0),        -- in
     WrCE     => Bus2IP_WrCE(0),        -- in
     DataIn   => Bus2IP_Data(29 to 31), -- in
     DataOut  => cr0RdData,             -- out
     RegData  => Cr0RegData             -- out
    );
  
  TP0_I :  entity xps_ll_temac_v1_01_b.reg_tp(imp)
    port map
    (
     Clk      => CLK,	                -- in
     RST      => RawReset,	        -- in
     RdCE     => Bus2IP_RdCE(1),	-- in
     WrCE     => Bus2IP_WrCE(1),	-- in
     DataIn   => Bus2IP_Data(16 to 31), -- in
     DataOut  => tp0RdData,	        -- out
     RegData  => Tp0RegData,	        -- out
     TPReq    => TPReq0		        -- out
    );
   
  IFGP0_I :  entity xps_ll_temac_v1_01_b.reg_ifgp(imp)
    port map
    (
     Clk      => CLK,	                -- in
     RST      => RawReset,	        -- in
     RdCE     => Bus2IP_RdCE(2),	-- in
     WrCE     => Bus2IP_WrCE(2),	-- in
     DataIn   => Bus2IP_Data(24 to 31), -- in
     DataOut  => ifgp0RdData,	        -- out
     RegData  => Ifgp0RegData	        -- out
    );				   
  
  IS0_I :  entity xps_ll_temac_v1_01_b.reg_is(imp)
    port map
    (
     Clk      => CLK,	                -- in
     RST      => RawReset,	        -- in
     RdCE     => Bus2IP_RdCE(3),	-- in
     WrCE     => Bus2IP_WrCE(3),	-- in
     Intrpts  => Intrpts0,	        -- in
     DataIn   => Bus2IP_Data(24 to 31), -- out
     DataOut  => is0RdData,	        -- out
     RegData  => is0RegData_i	        -- out
    );
  
  IP0_I :  entity xps_ll_temac_v1_01_b.reg_ip(imp)
    port map
    (
     Clk      => CLK,	            -- in
     RST      => RawReset,	    -- in
     RdCE     => Bus2IP_RdCE(4),    -- in
     IsIn     => is0RegData_i,	    -- in
     IeIn     => ie0RegData_i,	    -- in
     DataOut  => ip0RdData,	    -- out
     RegData  => Ip0RegData,	    -- out
     Intrpt   => Intrpt0	    -- out
    );
  
  IE0_I :  entity xps_ll_temac_v1_01_b.reg_ie(imp)
    port map
    (
     Clk      => CLK,	                -- in
     RST      => RawReset,	        -- in
     RdCE     => Bus2IP_RdCE(5),	-- in
     WrCE     => Bus2IP_WrCE(5),	-- in
     DataIn   => Bus2IP_Data(24 to 31), -- in
     DataOut  => ie0RdData,	        -- out
     RegData  => ie0RegData_i	        -- out
    );				    

DUAL_SYS: if(C_TEMAC1_ENABLED = 1) generate
begin

  RD_ACK_BLOCKER_PROCESS : process (CLK,RawReset)
  begin
    if (CLK'event and CLK = '1') then
      if (RawReset = '1') then
        rdAckBlocker <= '0';
      else
        rdAckBlocker <= ((dCR_ack_i and (dCR_Read0 or dCR_Read1)) or softRead0_d1 or softRead1_d1) or -- set when = '1'
                        (rdAckBlocker and -- hold  when = '1'
                        ((dCR_ack_i and (dCR_Read0 or dCR_Read1)) or softRead0_d1 or softRead1_d1)); -- clear when = '0'
      end if;
    end if;
  end process;

  WR_ACK_BLOCKER_PROCESS : process (CLK,RawReset)
  begin
    if (CLK'event and CLK = '1') then
      if (RawReset = '1') then
        wrAckBlocker <= '0';
      else
        wrAckBlocker <= ((dCR_ack_i and (dCR_Write0 or dCR_Write1)) or softWrite0_d1 or softWrite1_d1) or -- set when = '1'
                        (wrAckBlocker and -- hold  when = '1'
                        ((dCR_ack_i and (dCR_Write0 or dCR_Write1)) or softWrite0_d1 or softWrite1_d1)); -- clear when = '0'
      end if;
    end if;
  end process;

  --------------------------------------------------------------------------
  -- ACK_PROCESS
  --------------------------------------------------------------------------
  ACK_PROCESS : process (RawReset, CLK)
  begin
    if (RawReset = '1') then
      dCR_Read0_d1  <= '0';
      dCR_Write0_d1 <= '0';
      dCR_Read1_d1  <= '0';
      dCR_Write1_d1 <= '0';
      softRead0_d1  <= '0';
      softWrite0_d1 <= '0';
      softRead1_d1  <= '0';
      softWrite1_d1 <= '0';
      iP2Bus_WrAck_i  <= '0';
      iP2Bus_RdAck_i  <= '0';
    elsif (CLK'event and CLK = '1') then
      dCR_Read0_d1  <= dCR_Read0;
      dCR_Write0_d1 <= dCR_Write0;
      dCR_Read1_d1  <= dCR_Read1;
      dCR_Write1_d1 <= dCR_Write1;
      softRead0_d1  <= softRead0;
      softWrite0_d1 <= softWrite0;
      softRead1_d1  <= softRead1;
      softWrite1_d1 <= softWrite1;
      iP2Bus_WrAck_i<= ((dCR_ack_i and (dCR_Write0 or dCR_Write1)) or
                       softWrite0_d1 or softWrite1_d1) and not(wrAckBlocker);
      iP2Bus_RdAck_i<= ((dCR_ack_i and (dCR_Read0 or dCR_Read1)) or
                       softRead0_d1  or softRead1_d1) and not(rdAckBlocker);
    end if;
  end process;
  
  DCR_Clk     <= CLK;
  dCR_Read_i    <= dCR_Read0 or dCR_Read1;
  dCR_Read0   <= Bus2IP_RdCE(8) or
                 Bus2IP_RdCE(9) or
                 Bus2IP_RdCE(10)or
                 Bus2IP_RdCE(11);
  dCR_Read1   <= Bus2IP_RdCE(24)or
                 Bus2IP_RdCE(25)or
                 Bus2IP_RdCE(26)or
                 Bus2IP_RdCE(27);
  softRead0   <= Bus2IP_RdCE(0) or
                 Bus2IP_RdCE(1) or
                 Bus2IP_RdCE(2) or
                 Bus2IP_RdCE(3) or
                 Bus2IP_RdCE(4) or
                 Bus2IP_RdCE(5);
  softRead1   <= Bus2IP_RdCE(16) or
                 Bus2IP_RdCE(17) or
                 Bus2IP_RdCE(18) or
                 Bus2IP_RdCE(19) or
                 Bus2IP_RdCE(20) or
                 Bus2IP_RdCE(21);
  dCR_Write_i   <= dCR_Write0 or dCR_Write1;
  dCR_Write0  <= Bus2IP_WrCE(8) or
                 Bus2IP_WrCE(9) or
                 Bus2IP_WrCE(10)or
                 Bus2IP_WrCE(11);
  dCR_Write1  <= Bus2IP_WrCE(24)or
                 Bus2IP_WrCE(25)or
                 Bus2IP_WrCE(26)or
                 Bus2IP_WrCE(27);
  softWrite0  <= Bus2IP_WrCE(0) or
                 Bus2IP_WrCE(1) or
                 Bus2IP_WrCE(2) or
                 Bus2IP_WrCE(3) or
                 Bus2IP_WrCE(4) or
                 Bus2IP_WrCE(5);
  softWrite1  <= Bus2IP_WrCE(16) or
                 Bus2IP_WrCE(17) or
                 Bus2IP_WrCE(18) or
                 Bus2IP_WrCE(19) or
                 Bus2IP_WrCE(20) or
                 Bus2IP_WrCE(21);
  dcrTemac1Op <= dCR_Read1 or dCR_Write1;
  dCR_ABus_i    <= "0000000" & dcrTemac1Op & Bus2IP_Addr(28 to 29);
  dCR_ack_i   <= DCR_Ack;
  
  dCRTemac_DBus_i   <= Bus2IP_Data;
  temacDcr_DBus_i <= TemacDcr_DBus;
  
  Is1RegData <= is1RegData_i;
  Ie1RegData <= ie1RegData_i;

  CR1_I :  entity xps_ll_temac_v1_01_b.reg_cr(imp)
    port map
    (
     Clk      => CLK,                   -- in
     RST      => RawReset,              -- in
     RawReset => RawReset,              -- in
     RdCE     => Bus2IP_RdCE(16),       -- in
     WrCE     => Bus2IP_WrCE(16),       -- in
     DataIn   => Bus2IP_Data(29 to 31), -- in
     DataOut  => cr1RdData,             -- out
     RegData  => Cr1RegData             -- out
    );
  
  TP1_I :  entity xps_ll_temac_v1_01_b.reg_tp(imp)
    port map
    (
     Clk      => CLK,	                -- in
     RST      => RawReset,	        -- in
     RdCE     => Bus2IP_RdCE(17),	-- in
     WrCE     => Bus2IP_WrCE(17),	-- in
     DataIn   => Bus2IP_Data(16 to 31), -- in
     DataOut  => tp1RdData,	        -- out
     RegData  => Tp1RegData,	        -- out
     TPReq    => TPReq1		        -- out
    );
   
  IFGP1_I :  entity xps_ll_temac_v1_01_b.reg_ifgp(imp)
    port map
    (
     Clk      => CLK,	                -- in
     RST      => RawReset,	        -- in
     RdCE     => Bus2IP_RdCE(18),	-- in
     WrCE     => Bus2IP_WrCE(18),	-- in
     DataIn   => Bus2IP_Data(24 to 31), -- in
     DataOut  => ifgp1RdData,	        -- out
     RegData  => Ifgp1RegData	        -- out
    );				   
  
  IS1_I :  entity xps_ll_temac_v1_01_b.reg_is(imp)
    port map
    (
     Clk      => CLK,	                -- in
     RST      => RawReset,	        -- in
     RdCE     => Bus2IP_RdCE(19),	-- in
     WrCE     => Bus2IP_WrCE(19),	-- in
     Intrpts  => Intrpts1,	        -- in
     DataIn   => Bus2IP_Data(24 to 31), -- out
     DataOut  => is1RdData,	        -- out
     RegData  => is1RegData_i	        -- out
    );
  
  IP1_I :  entity xps_ll_temac_v1_01_b.reg_ip(imp)
    port map
    (
     Clk      => CLK,	            -- in
     RST      => RawReset,	    -- in
     RdCE     => Bus2IP_RdCE(20),   -- in
     IsIn     => is1RegData_i,	    -- in
     IeIn     => ie1RegData_i,	    -- in
     DataOut  => ip1RdData,	    -- out
     RegData  => Ip1RegData,	    -- out
     Intrpt   => Intrpt1	    -- out
    );
  
  IE1_I :  entity xps_ll_temac_v1_01_b.reg_ie(imp)
    port map
    (
     Clk      => CLK,	                -- in
     RST      => RawReset,	        -- in
     RdCE     => Bus2IP_RdCE(21),	-- in
     WrCE     => Bus2IP_WrCE(21),	-- in
     DataIn   => Bus2IP_Data(24 to 31), -- in
     DataOut  => ie1RdData,	        -- out
     RegData  => ie1RegData_i	        -- out
    );				    

  rdData  <= (
              ("00000000000000000000000000000" & cr0RdData) or
              ("0000000000000000" & tp0RdData) or
              ("000000000000000000000000" & ifgp0RdData)or
              ("000000000000000000000000" & is0RdData) or
              ("000000000000000000000000" & ie0RdData) or
              ("000000000000000000000000" & ip0RdData) or
              ("00000000000000000000000000000" & cr1RdData) or
              ("0000000000000000" & tp1RdData) or
              ("000000000000000000000000" & ifgp1RdData)or
              ("000000000000000000000000" & is1RdData) or
              ("000000000000000000000000" & ie1RdData) or
              ("000000000000000000000000" & ip1RdData)

             );
  IP2Bus_Data <= rdData when (dCR_Read0 = '0' and dCR_Read1 = '0') else
                 temacDcr_DBus_i;

end generate DUAL_SYS;

SINGLE_SYS: if(C_TEMAC1_ENABLED = 0) generate
begin

  RD_ACK_BLOCKER_PROCESS : process (CLK,RawReset)
  begin
    if (CLK'event and CLK = '1') then
      if (RawReset = '1') then
        rdAckBlocker <= '0';
      else
        rdAckBlocker <= ((dCR_ack_i and dCR_Read0) or softRead0_d1) or -- set when = '1'
                        (rdAckBlocker and -- hold  when = '1'
                        ((dCR_ack_i and dCR_Read0) or softRead0_d1)); -- clear when = '0'
      end if;
    end if;
  end process;

  WR_ACK_BLOCKER_PROCESS : process (CLK,RawReset)
  begin
    if (CLK'event and CLK = '1') then
      if (RawReset = '1') then
        wrAckBlocker <= '0';
      else
        wrAckBlocker <= ((dCR_ack_i and dCR_Write0) or softWrite0_d1) or -- set when = '1'
                        (wrAckBlocker and -- hold  when = '1'
                        ((dCR_ack_i and dCR_Write0) or softWrite0_d1)); -- clear when = '0'
      end if;
    end if;
  end process;

  --------------------------------------------------------------------------
  -- ACK_PROCESS
  --------------------------------------------------------------------------
  ACK_PROCESS : process (RawReset, CLK)
  begin
    if (RawReset = '1') then
      dCR_Read0_d1  <= '0';
      dCR_Write0_d1 <= '0';
      softRead0_d1  <= '0';
      softWrite0_d1 <= '0';
      iP2Bus_WrAck_i  <= '0';
      iP2Bus_RdAck_i  <= '0';
    elsif (CLK'event and CLK = '1') then
      dCR_Read0_d1  <= dCR_Read0;
      dCR_Write0_d1 <= dCR_Write0;
      softRead0_d1  <= softRead0;
      softWrite0_d1 <= softWrite0;
      iP2Bus_WrAck_i  <= ((dCR_ack_i and dCR_Write0) or softWrite0_d1) and not(wrAckBlocker);
      iP2Bus_RdAck_i  <= ((dCR_ack_i and dCR_Read0)  or softRead0_d1) and not(rdAckBlocker);
    end if;
  end process;

  dCR_Read1_d1  <= '0';
  dCR_Write1_d1 <= '0';

  TPReq1	<= '0';
  Intrpt1  	<= '0';
  Is1RegData    <= (others => '0');
  Ie1RegData    <= (others => '0');
  Cr1RegData  	<= (others => '0');
  Tp1RegData  	<= (others => '0');
  Ifgp1RegData	<= (others => '0');
  Is1RegData  	<= (others => '0');
  Ip1RegData  	<= (others => '0');
  Ie1RegData 	<= (others => '0');

  DCR_Clk     <= CLK;
  dCR_Read_i    <= dCR_Read0;
  dCR_Read0   <= Bus2IP_RdCE(8) or
                 Bus2IP_RdCE(9) or
                 Bus2IP_RdCE(10)or
                 Bus2IP_RdCE(11);
  dCR_Read1   <= '0';
  softRead0   <= Bus2IP_RdCE(0) or
                 Bus2IP_RdCE(1) or
                 Bus2IP_RdCE(2) or
                 Bus2IP_RdCE(3) or
                 Bus2IP_RdCE(4) or
                 Bus2IP_RdCE(5);
  softRead1   <= '0';
  dCR_Write_i   <= dCR_Write0;
  dCR_Write0  <= Bus2IP_WrCE(8) or
                 Bus2IP_WrCE(9) or
                 Bus2IP_WrCE(10)or
                 Bus2IP_WrCE(11);
  dCR_Write1  <= '0';
  softWrite0  <= Bus2IP_WrCE(0) or
                 Bus2IP_WrCE(1) or
                 Bus2IP_WrCE(2) or
                 Bus2IP_WrCE(3) or
                 Bus2IP_WrCE(4) or
                 Bus2IP_WrCE(5);
  softWrite1  <= '0';
  dcrTemac1Op <= '0';
  dCR_ABus_i    <= "00000000" & Bus2IP_Addr(28 to 29);
  dCR_ack_i   <= DCR_Ack;
  
  dCRTemac_DBus_i   <= Bus2IP_Data;
  temacDcr_DBus_i <= TemacDcr_DBus;

  rdData  <= (
              ("00000000000000000000000000000" & cr0RdData) or
              ("0000000000000000" & tp0RdData) or
              ("000000000000000000000000" & ifgp0RdData)or
              ("000000000000000000000000" & is0RdData) or
              ("000000000000000000000000" & ie0RdData) or
              ("000000000000000000000000" & ip0RdData)
             );
  IP2Bus_Data <= rdData when (dCR_Read0 = '0') else
                 temacDcr_DBus_i;

end generate SINGLE_SYS;

end imp;
