------------------------------------------------------------------------------
-- $Id: rx_temac_if_sm.vhd,v 1.1 2008/02/08 04:15:11 shurt Exp $
------------------------------------------------------------------------------
-- rx_sm.vhd            
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        temac_if_sm.vhd
-- Version:         v1.00a
-- Description:     Controls data flow from the client interface to the
--                  receive status fifo and receive data fifo
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the 
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DRP
-- History:
--  DCW      2006.05.03      -- First version
--
--  <initials>      <date> 
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Port Declaration
------------------------------------------------------------------------------

entity rx_temac_if_sm is
    generic (
             C_RESET_ACTIVE  : std_logic := '1'
            );

    port    (
             llTakeRxStatsNow_d1   : in  std_logic;
             LocalLinkPaused       : in  std_logic;
             PauseLocalLinkFull    : out std_logic;
             FifoFullReset         : out std_logic;
             AbortWrite            : out std_logic;
             SM_encoded            : out  std_logic_vector(0 to 2);
             
             LLTemac_Clk           : in  std_logic;
             LLTemac_Rst           : in  std_logic;
             
             RXFIFO2IP_Almost_Full : in  std_logic;
             RXFIFO2IP_Full        : in  std_logic;
             RXFIFO2IP_Ovr         : in  std_logic;
             RXFIFO2LL_Und         : in  std_logic;
             
             RSFIFO2IP_Full        : in  std_logic;
             RSFIFO2IP_Ovr         : in  std_logic;
             RSFIFO2LL_Und         : in  std_logic;
             Rx_Csum_Done          : in  std_logic;
             
             Rx_Byte_Cnt_Rst       : out std_logic;
             Rx2Cl_IF_RdEn         : out std_logic;
             RxD_Reg_CE            : out std_logic;
             
             Cl_IF2Rx_F_Mt         : in  std_logic;
             Cl_If2Rx_End_Of_Pckt  : in  std_logic;
             Cl_IF2Rx_EndOfFrameMarker     : in  std_logic;
             Cl_IF2Rx_FrameIsCorruptMarker : in  std_logic;
             Cl_If2Rx_BadFrame     : in  std_logic;
             Cl_If2Rx_FrameDrop    : in  std_logic;
             Cl_If2Rx_BrdCast      : in  std_logic;
             Cl_If2Rx_MulCast      : in  std_logic;
             Cl_If2Rx_GoodFrame    : in  std_logic;
             
             RegCR_BrdCast_Rej     : in  std_logic;
             RegCR_MulCast_Rej     : in  std_logic;
             Rx_Cmplt              : out std_logic;
             Rx_Pckt_Rej           : out std_logic;
             Pckt_Ovr_Run          : out std_logic;
             
             Rx_Sop                : out std_logic;
             Rx_Eop                : out std_logic;
             Csum_Eop              : out std_logic;
             
             Rx2Cl_IF_Rd_Frm_Inds  : out std_logic;
             Rx2Cl_IF_Rd_Frm_Stats : out std_logic;
             
             Cl_If2Rx_StatsVld	   : in  std_logic;

             Good_Bad_n            : out std_logic_vector(0 to 5);
             Last_Word             : out std_logic
            );
            
end rx_temac_if_sm;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

architecture beh of rx_temac_if_sm is

------------------------------------------------------------------------------
-- Type Declarations
------------------------------------------------------------------------------

type RX_SM_TYPE is (
                    IDLE,
                    SOP,
                    RD_CLFIFO,
                    QUALIFY_PCKT,
                    BAD_PCKT,
                    CLR_PKT_STATUS,
                    EOP,
                    FLUSH_CL_FIFO
                   );

------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------

signal rx_sm_ps            : RX_SM_TYPE;
signal rx_sm_ns            : RX_SM_TYPE;

signal pckt_Ovr_Run_i      : std_logic;
signal pckt_Ovr_Run_Rst    : std_logic;

signal rxFIFO2IP_Full_d1   : std_logic;
signal rx_cmplt_i          : std_logic;

signal abortWrite_i        : std_logic;
signal FifoFullReset_i     : std_logic;
signal FifoFullReset_d1    : std_logic;
signal FifoFullReset_d2    : std_logic;
signal FifoFullReset_d3    : std_logic;
signal FifoFullReset_d4    : std_logic;
signal FifoFullReset_d5    : std_logic;
signal FifoFullReset_d6    : std_logic;
signal cl_If2Rx_BadFrame_d1: std_logic; -- delay to align with multicast/broadcast indicator
signal cl_If2Rx_GoodFrame_d1: std_logic; -- delay to align with multicast/broadcast indicator
signal llTakeRxStatsNow_d2 : std_logic;

begin
  stretch_status : process ( LLTemac_Clk, LLTemac_Rst )
  begin
    if ((LLTemac_Rst = C_RESET_ACTIVE) or (FifoFullReset_d6 = '1')) then
      cl_If2Rx_BadFrame_d1  <= '0';
      cl_If2Rx_GoodFrame_d1 <= '0';
      llTakeRxStatsNow_d2   <= '0';
    elsif (LLTemac_Clk'event and LLTemac_Clk = '1') then
      llTakeRxStatsNow_d2  <= llTakeRxStatsNow_d1;
      cl_If2Rx_BadFrame_d1 <= Cl_If2Rx_BadFrame or      -- set   - indicator from rx_cl_if
                             (cl_If2Rx_BadFrame_d1 and  -- hold  - until stats ready
                             not llTakeRxStatsNow_d2);  -- clear - stats ready
      cl_If2Rx_GoodFrame_d1 <= Cl_If2Rx_GoodFrame or    -- set   - indicator from rx_cl_if
                             (cl_If2Rx_GoodFrame_d1 and -- hold  - until stats ready
                             not llTakeRxStatsNow_d2);  -- clear - stats ready
    end if;
  end process;

FifoFullReset <= FifoFullReset_d1 or FifoFullReset_d2 or FifoFullReset_d3 or FifoFullReset_d4 or FifoFullReset_d5 or FifoFullReset_d6 or FifoFullReset_i;        

StretchFifoReset : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk)) then
         if(LLTemac_Rst='1') then
            FifoFullReset_d1 <= '0';
            FifoFullReset_d2 <= '0';
            FifoFullReset_d3 <= '0';
            FifoFullReset_d4 <= '0';
            FifoFullReset_d5 <= '0';
            FifoFullReset_d6 <= '0';
         else
            FifoFullReset_d1 <= FifoFullReset_i;
            FifoFullReset_d2 <= FifoFullReset_d1;
            FifoFullReset_d3 <= FifoFullReset_d2;
            FifoFullReset_d4 <= FifoFullReset_d3;
            FifoFullReset_d5 <= FifoFullReset_d4;
            FifoFullReset_d6 <= FifoFullReset_d5;
         end if;
      end if;
   end process;

------------------------------------------------------------------------------
-- Concurrent Assignments
------------------------------------------------------------------------------
Pckt_Ovr_Run <= pckt_Ovr_Run_i;
Rx_cmplt     <= rx_cmplt_i;
FORCE_ABORT_WRITE_RX_DATA_FIFO : process (LLTemac_Clk)
begin
    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
        if (LLTemac_Rst = C_RESET_ACTIVE) then
            abortWrite_i <= '0';
            AbortWrite   <= '0';
        elsif (rx_sm_ps = BAD_PCKT) then
            abortWrite_i <= '1';
            AbortWrite   <= abortWrite_i;
        else
            abortWrite_i <= '0';
            AbortWrite   <= abortWrite_i;
        end if;
    end if;
end process;

------------------------------------------------------------------------------
-- LATCH_PCKT_FIFO_FULL_PROCESS
------------------------------------------------------------------------------

LATCH_PCKT_FIFO_FULL_PROCESS : process (LLTemac_Clk)
begin
  if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    if (LLTemac_Rst = C_RESET_ACTIVE) then
      rxFIFO2IP_Full_d1 <= '0';
    else
      rxFIFO2IP_Full_d1 <= RXFIFO2IP_Full or                     -- set   indicator from rx packet FIFO
                          (rxFIFO2IP_Full_d1 and                 -- hold  until end of packet
                          not (rx_cmplt_i or pckt_Ovr_Run_Rst)); -- clear at end of packet and while Idle
    end if;
  end if;
end process;

------------------------------------------------------------------------------
-- PCKT_OVR_RUN_REG_PROCESS
------------------------------------------------------------------------------

PCKT_OVR_RUN_REG_PROCESS : process (LLTemac_Clk)
begin
    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
        if (LLTemac_Rst = C_RESET_ACTIVE) or
           (pckt_Ovr_Run_Rst = '1') then
            pckt_Ovr_Run_i <= '0';
        else
            pckt_Ovr_Run_i <= RXFIFO2IP_Ovr;
        end if;
    end if;
end process;

------------------------------------------------------------------------------
-- RX State Machine
-- RX_SM_SYNC_PROCESS: synchronous process of the state machine
-- RX_SM_CMB_PROCESS:  combinatorial next-state logic
------------------------------------------------------------------------------
-- This state machine determines the current state of the Transmitter
------------------------------------------------------------------------------

RX_SM_SYNC_PROCESS : process ( LLTemac_Clk )
begin
    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
        if (LLTemac_Rst = C_RESET_ACTIVE) then
            rx_sm_ps <= IDLE;
        else
            rx_sm_ps <= rx_sm_ns;
        end if;
    end if;
end process;

RX_SM_CMB_PROCESS : process (
                             rx_sm_ps,
                             RXFIFO2IP_Full,
                             RXFIFO2IP_Ovr,
                             RXFIFO2LL_Und,
                             RSFIFO2IP_Ovr,
                             RSFIFO2LL_Und,
                             RSFIFO2IP_Full,
                             Cl_IF2Rx_F_Mt,
                             Cl_If2Rx_End_Of_Pckt,
                             Cl_If2Rx_BadFrame_d1,
--                             Cl_If2Rx_FrameDrop,
                             Cl_IF2Rx_EndOfFrameMarker,
                             Cl_IF2Rx_FrameIsCorruptMarker,
                             Cl_If2Rx_BrdCast,
                             Cl_If2Rx_MulCast,
                             Cl_If2Rx_GoodFrame_d1,
                             RegCR_BrdCast_Rej,
                             RegCR_MulCast_Rej,
                             Rx_Csum_Done,
                             RXFIFO2IP_Almost_Full,
                             rxFIFO2IP_Full_d1,
                             pckt_Ovr_Run_i,
                             llTakeRxStatsNow_d1,
                             LocalLinkPaused
                             )
begin
    PauseLocalLinkFull    <= '0';
    Rx2Cl_IF_RdEn         <= '0';
    rx_cmplt_i            <= '0';
    Rx_Byte_Cnt_Rst       <= '0';
    RxD_Reg_CE            <= '0';
    Rx2Cl_IF_Rd_Frm_Inds  <= '0';
    Rx2Cl_IF_Rd_Frm_Stats <= '0';
    pckt_Ovr_Run_Rst      <= '0';
    Rx_Sop                <= '0';
    Rx_Eop                <= '0';
    Csum_Eop              <= '0';
    Good_Bad_n            <= (others=>'1');
    Last_Word             <= '0';
    Rx_Pckt_Rej           <= '0';
    FifoFullReset_i       <= '0';
    case rx_sm_ps is

        when IDLE =>
            SM_encoded <= "000";
            Rx_Byte_Cnt_Rst  <='1';
            pckt_Ovr_Run_Rst <='1';
            if ((RXFIFO2IP_Ovr='1') or
                (RXFIFO2LL_Und='1') or 
                (RXFIFO2IP_Full='1') or
                (RXFIFO2IP_Almost_Full = '1') or
                (rxFIFO2IP_Full_d1='1') or
                (RSFIFO2IP_Ovr='1') or
                (RSFIFO2LL_Und='1') or
                (RSFIFO2IP_Full='1')) then

                rx_sm_ns <= IDLE;
            elsif (Cl_IF2Rx_F_Mt = '0') then
                rx_sm_ns <= SOP;
            else
                Rx2Cl_IF_Rd_Frm_Inds  <= '1';
                rx_sm_ns <= IDLE;
            end if;
                  
        when SOP =>
            SM_encoded <= "001";
            Rx_Sop <= '1';
            Rx2Cl_IF_RdEn <= not Cl_IF2Rx_F_Mt;
            RxD_Reg_CE    <= not Cl_IF2Rx_F_Mt;            
            if(RXFIFO2IP_Almost_Full='1') then
               rx_sm_ns <= BAD_PCKT;
               Rx_Pckt_Rej <= not ((Cl_If2Rx_BrdCast and RegCR_BrdCast_Rej) or
                                (Cl_If2Rx_MulCast and RegCR_MulCast_Rej));
            else
               rx_sm_ns <= RD_CLFIFO;
            end if;         

        when RD_CLFIFO =>
            SM_encoded <= "010";
            if(Cl_IF2Rx_FrameIsCorruptMarker='1' or RXFIFO2IP_Almost_Full='1') then
              rx_sm_ns <= BAD_PCKT;
              Rx_Pckt_Rej <= not ((Cl_If2Rx_BrdCast and RegCR_BrdCast_Rej) or
                                (Cl_If2Rx_MulCast and RegCR_MulCast_Rej));
            elsif(Cl_IF2Rx_EndOfFrameMarker='1') then
              rx_sm_ns <= QUALIFY_PCKT;
              Csum_Eop <= '1';
            else
              Rx2Cl_IF_RdEn <= not Cl_IF2Rx_F_Mt;
              RxD_Reg_CE    <= not Cl_IF2Rx_F_Mt;
              rx_sm_ns <= RD_CLFIFO;
            end if;
            
        when QUALIFY_PCKT =>
            SM_encoded <= "011";
            if((cl_If2Rx_BadFrame_d1='1' and llTakeRxStatsNow_d1 = '1') or
--               (Cl_If2Rx_FrameDrop='1') or
               (pckt_Ovr_Run_i='1') or
               (RXFIFO2IP_Almost_Full='1') or
               (RXFIFO2IP_Full='1') or                
               (rxFIFO2IP_Full_d1='1')) or-- because a write actually takes place after this decision point so we need at least one more entry available
              ((Cl_If2Rx_BrdCast='1') and (RegCR_BrdCast_Rej = '1') and llTakeRxStatsNow_d1 = '1') or
              ((Cl_If2Rx_MulCast='1') and (RegCR_MulCast_Rej = '1') and llTakeRxStatsNow_d1 = '1') or
              (RSFIFO2IP_Full='1') then
               rx_sm_ns <= BAD_PCKT;
               Rx_Pckt_Rej <= not ((Cl_If2Rx_BrdCast and RegCR_BrdCast_Rej and llTakeRxStatsNow_d1) or
                                (Cl_If2Rx_MulCast and RegCR_MulCast_Rej and llTakeRxStatsNow_d1));
            elsif(cl_If2Rx_GoodFrame_d1 = '1' and llTakeRxStatsNow_d1 = '1') then
                rx_sm_ns <= CLR_PKT_STATUS;
            else
                rx_sm_ns <= QUALIFY_PCKT;
            end if;                       
            
        when BAD_PCKT =>
            SM_encoded <= "100";
                    PauseLocalLinkFull    <= '1';
                    if (LocalLinkPaused = '1') then
                      FifoFullReset_i <= '1';
                      rx_sm_ns <= IDLE;                    
                    else
                      rx_sm_ns <= BAD_PCKT;
                    end if;            
--            rx_sm_ns <= CLR_PKT_STATUS;
--            Csum_Eop <= '1';
            Good_Bad_n <= (others=>'0');            
--            Rx_Pckt_Rej <= not ((Cl_If2Rx_BrdCast and RegCR_BrdCast_Rej) or
--                                (Cl_If2Rx_MulCast and RegCR_MulCast_Rej));
            
        when CLR_PKT_STATUS =>
            SM_encoded <= "101";
            rx_sm_ns <= EOP;
            Rx2Cl_IF_Rd_Frm_Inds <= '1';
            Rx2Cl_IF_Rd_Frm_Stats <= '1';
            Last_Word <= '1';
               
        when EOP =>
            SM_encoded <= "110";
            Rx_Eop <= '1';
            rx_cmplt_i <= '1';               
            if(Cl_IF2Rx_EndOfFrameMarker='0') then
              Rx2Cl_IF_RdEn <= not Cl_IF2Rx_F_Mt;
              rx_sm_ns <= FLUSH_CL_FIFO;
              Rx_Pckt_Rej <= not ((Cl_If2Rx_BrdCast and RegCR_BrdCast_Rej) or
                                (Cl_If2Rx_MulCast and RegCR_MulCast_Rej));
            elsif(Rx_Csum_Done='1') then
              rx_sm_ns <= IDLE;
            else
              rx_sm_ns <= EOP;
            end if;

        when FLUSH_CL_FIFO =>
            SM_encoded <= "111";
--            Rx_Pckt_Rej <= not ((Cl_If2Rx_BrdCast and RegCR_BrdCast_Rej) or
--                                (Cl_If2Rx_MulCast and RegCR_MulCast_Rej));
            if(Cl_IF2Rx_EndOfFrameMarker='0') then
              Rx2Cl_IF_RdEn <= not Cl_IF2Rx_F_Mt;
              rx_sm_ns <= FLUSH_CL_FIFO;
            else
              rx_sm_ns <= IDLE;
            end if;

        when others   =>  -- default to IDLE
            SM_encoded <= "000";
            rx_sm_ns <= IDLE;

    end case;
end process;


end beh;
