------------------------------------------------------------------------------
-- $Id: tx_llink_top.vhd,v 1.3 2008/06/19 21:41:01 mwelter Exp $
------------------------------------------------------------------------------
-- tx_llink_top.vhd
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        tx_llink_top.vhd
-- Version:         v1.00a
-- Description:     This is the transmit interface between LL and the hard
--                  Temac.
--                  
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DRP
-- History:
--  DRP      2006.05.02      -- First version
--
--  <initials>      <date>
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x"
--      reset signals:                          "rst", "rst_n"
--      generics:                               "C_*"
--      user defined types:                     "*_TYPE"
--      state machine next state:               "*_ns"
--      state machine current state:            "*_cs"
--      combinatorial signals:                  "*_cmb"
--      pipelined or register delay signals:    "*_d#"
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce"
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin"
--      ports:                                  - Names begin with Uppercase
--      processes:                              "*_PROCESS"
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.conv_integer;

library proc_common_v2_00_a;
use proc_common_v2_00_a.coregen_comp_defs.all;
use proc_common_v2_00_a.proc_common_pkg.log2;

-- synopsys translate_off
library XilinxCoreLib;
-- synopsys translate_on

library xps_ll_temac_v1_01_b;
use xps_ll_temac_v1_01_b.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Port Declaration
------------------------------------------------------------------------------

entity tx_llink_top is
   generic(
      C_RESET_ACTIVE  : std_logic :=  '1';
      C_TEMAC_TXCSUM  : integer   :=    0;
      C_CLIENT_DWIDTH : integer   :=    8;
      C_TEMAC_TXFIFO  : integer   := 4096 
      );

   port(
      LLTemac_Clk             : in  std_logic;
      LLTemac_Rst             : in  std_logic;
      LLTemac_Data            : in  std_logic_vector(0 to 31);
      LLTemac_SOF_n           : in  std_logic;
      LLTemac_SOP_n           : in  std_logic;
      LLTemac_EOF_n           : in  std_logic;
      LLTemac_EOP_n           : in  std_logic;
      LLTemac_SRC_RDY_n       : in  std_logic;
      LLTemac_REM             : in  std_logic_vector(0 to 3);
      LLTemac_DST_RDY_n       : out std_logic;             

      TXFIFO_Und_Intr         : out std_logic;

      Tx2ClientPauseReq       : in  std_logic;
      ClientEmacPauseReq      : out std_logic;

      Tx_cmplt                : out std_logic;

      Tx_Cl_Clk               : in  std_logic;
      ClientEmacTxd           : out std_logic_vector(7 downto 0);
      ClientEmacTxdVld        : out std_logic;
      ClientEmacTxdVldMsw     : out std_logic;
      ClientEmacTxFirstByte   : out std_logic;
      ClientEmacTxUnderRun    : out std_logic;
      EmacClientTxAck         : in  std_logic;
      EmacClientTxCollision   : in  std_logic;
      EmacClientTxRetransmit  : in  std_logic;
      
      Tx2ClientUnderRunIntrpt : out std_logic
      );
      
end tx_llink_top;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

architecture beh of tx_llink_top is

attribute KEEP : string;
attribute KEEP of Tx_Cl_Clk : signal is "TRUE";

------------------------------------------------------------------------------
-- Constant Declarations
------------------------------------------------------------------------------

constant FIFO_DEPTH_LOG2X : integer := 13;
                                            
------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------

signal lL2CSFIFO_data         : std_logic_vector(0 to 35);
signal lL2CSFIFO_wren         : std_logic;
signal lL2CSFIFO_wren_dly1    : std_logic;
signal cSFIFO2LL_Full         : std_logic;
signal iP2CSFIFO_RdReq        : std_logic;
signal cSFIFO2IP_Empty        : std_logic;
signal cSFIFO2IP_Und          : std_logic;
signal cSFIFO2IP_Ovr          : std_logic;
signal cSFIFO2IP_Data         : std_logic_vector(0 to 35);
signal txFIFO2IP_RdAck        : std_logic;
signal lL2TXFIFO_Data         : std_logic_vector(0 to 35);
signal lL2TXFIFO_Wren         : std_logic;
signal ip2TXFIFO_RdReq        : std_logic;
signal txFIFO2IP_Data         : std_logic_vector(0 to 35);
signal txFIFO2LL_Full         : std_logic;
signal txFIFO2IP_Empty        : std_logic;
signal txFIFO2IP_Und          : std_logic;
signal txFIFO2IP_Ovr          : std_logic;
signal txFIFO2IP_aFull        : std_logic;
signal csum_ready             : std_logic;
signal lLTemac_DST_RDY_n_i    : std_logic;

signal clientEmacTxd_i         : std_logic_vector(7 downto 0);
signal clientEmacTxdVld_i      : std_logic;
signal clientEmacTxdVldMsw_i   : std_logic;
signal clientEmacTxFirstByte_i : std_logic;
signal clientEmacTxUnderRun_i  : std_logic;
signal tx_cmplt_i              : std_logic;
signal clientEmacPauseReq_i    : std_logic;
signal t_sM_encoded            : std_logic_vector(0 to 2);
signal sig_data_count          : std_logic_vector(log2(C_TEMAC_TXFIFO/4)-1 downto 0);

signal txFIFO2IP_aFull_cmb      : std_logic;
signal csfifo2ll_full_reg : std_logic;

begin

------------------------------------------------------------------------------
-- Concurrent Signal Assignments
------------------------------------------------------------------------------
ClientEmacTxd       	<= clientEmacTxd_i;
ClientEmacTxdVld    	<= clientEmacTxdVld_i;
ClientEmacTxdVldMsw 	<= clientEmacTxdVldMsw_i;
ClientEmacTxFirstByte	<= clientEmacTxFirstByte_i;
ClientEmacTxUnderRun	<= clientEmacTxUnderRun_i;
Tx_cmplt            	<= tx_cmplt_i;
ClientEmacPauseReq      <= clientEmacPauseReq_i;

cSFIFO2IP_Und <= cSFIFO2IP_Empty and iP2CSFIFO_RdReq;
cSFIFO2IP_Ovr <= cSFIFO2LL_Full and lL2CSFIFO_wren;
txFIFO2IP_Und <= txFIFO2IP_Empty and iP2TXFIFO_RdReq;
txFIFO2IP_Ovr <= txFIFO2LL_Full and lL2TXFIFO_Wren;
lLTemac_DST_RDY_n_i <= cSFIFO2LL_Full_reg or txFIFO2LL_Full or txFIFO2IP_aFull or not csum_ready;
LLTemac_DST_RDY_n <= lLTemac_DST_RDY_n_i;



I_TX_LL_IF : entity xps_ll_temac_v1_01_b.tx_ll_if 
   generic map(
      C_TEMAC_TXCSUM       => C_TEMAC_TXCSUM,
      C_FIFO_DEPTH_LOG2X   => FIFO_DEPTH_LOG2X
      )
   port map(
      LLTemac_Clk          => LLTemac_Clk,         -- in  
      LLTemac_Rst          => LLTemac_Rst,         -- in  
                       
      LLTemac_Data         => LLTemac_Data,        -- in  
      LLTemac_SOF_n        => LLTemac_SOF_n,       -- in  
      LLTemac_SOP_n        => LLTemac_SOP_n,       -- in  
      LLTemac_EOF_n        => LLTemac_EOF_n,       -- in  
      LLTemac_EOP_n        => LLTemac_EOP_n,       -- in  
      LLTemac_SRC_RDY_n    => LLTemac_SRC_RDY_n,   -- in  
      LLTemac_REM          => LLTemac_REM,         -- in  
      LLTemac_DST_RDY_n    => lLTemac_DST_RDY_n_i, -- in 
                        
      CSFIFO2LL_Full       => cSFIFO2LL_Full,      -- in  
      lL2CSFIFO_wren       => lL2CSFIFO_wren,      -- out 
      lL2CSFIFO_data       => lL2CSFIFO_data,      -- out 
                        
      TXFIFO2LL_Full       => txFIFO2LL_Full,      -- in  
      lL2TXFIFO_Data       => lL2TXFIFO_Data,      -- out 
      lL2TXFIFO_Wren       => lL2TXFIFO_Wren,      -- out 
      Csum_Ready           => csum_ready           -- in
      );

 I_TX_CSUM_FIFO : sync_fifo_v5_0
    generic map(
              c_dcount_width       => 9,
              c_enable_rlocs       => 0,
              c_has_dcount         => 0,
              c_has_rd_ack         => 1,
              c_has_rd_err         => 0,
              c_has_wr_ack         => 0,
              c_has_wr_err         => 0,
              c_memory_type        => 1,
              c_ports_differ       => 0,
              c_rd_ack_low         => 0,
              c_read_data_width    => 36,
              c_read_depth         => (C_TEMAC_TXFIFO/64),
              c_rd_err_low         => 0,
              c_wr_ack_low         => 0,
              c_wr_err_low         => 0,
              c_write_data_width   => 36,
              c_write_depth        => (C_TEMAC_TXFIFO/64)
         )
    port map(
              CLK          => LLTemac_Clk,           -- in
              SINIT        => LLTemac_Rst,           -- in
              DIN          => lL2CSFIFO_data,        -- in
              WR_EN        => lL2CSFIFO_wren,        -- in
              RD_EN        => iP2CSFIFO_RdReq,       -- in
              DOUT         => cSFIFO2IP_Data,        -- out
              FULL         => cSFIFO2LL_full,        -- out
              EMPTY        => cSFIFO2IP_Empty,       -- out
              RD_ACK       => txFIFO2IP_RdAck,       -- out
              WR_ACK       => open,                  -- out
              RD_ERR       => open,                  -- out
              WR_ERR       => open,                  -- out
              DATA_COUNT   => open                   -- out
         ); 

-- Start Async FIFO 6.1 model --------------------------------------                                                   

-- I_TX_CSUM_FIFO : async_fifo_v6_1 
--    generic map(
--         C_DATA_WIDTH       => 36,
--         C_ENABLE_RLOCS     =>  0,
--         C_FIFO_DEPTH       =>  (C_TEMAC_TXFIFO/64),
--         C_HAS_ALMOST_EMPTY =>  0,
--         C_HAS_ALMOST_FULL  =>  0,
--         C_HAS_RD_ACK       =>  1,
--         C_HAS_RD_COUNT     =>  0,
--         C_HAS_RD_ERR       =>  0,
--         C_HAS_WR_ACK       =>  0,
--         C_HAS_WR_COUNT     =>  0,
--         C_HAS_WR_ERR       =>  0,
--         C_RD_ACK_LOW       =>  0,
--         C_RD_COUNT_WIDTH   =>  9,
--         C_RD_ERR_LOW       =>  0,
--         C_USE_BLOCKMEM     =>  1,
--         C_WR_ACK_LOW       =>  0,
--         C_WR_COUNT_WIDTH   =>  9,
--         C_WR_ERR_LOW       =>  0   
--         )
--    port map(
--         Din            => lL2CSFIFO_data,        -- in 
--         Wr_en          => lL2CSFIFO_wren,        -- in 
--         Wr_clk         => LLTemac_Clk,           -- in 
--         Rd_en          => iP2CSFIFO_RdReq,       -- in 
--         Rd_clk         => LLTemac_Clk,           -- in 
--         Ainit          => LLTemac_Rst,           -- in 
--         Dout           => cSFIFO2IP_Data,        -- out
--         Full           => cSFIFO2LL_full,        -- out
--         Empty          => cSFIFO2IP_Empty,       -- out
--         Almost_full    => open,                  -- out
--         Almost_empty   => open,                  -- out
--         Wr_count       => open,                  -- out                           
--         Rd_count       => open,                  -- out                            
--         Rd_ack         => txFIFO2IP_RdAck,       -- out 
--         Rd_err         => open,                  -- out 
--         Wr_ack         => open,                  -- out 
--         Wr_err         => open                   -- out 
--         ); 

-- The following code must appear in the VHDL architecture
-- body. Substitute your own instance name and net names.

------------- Begin Cut here for INSTANTIATION Template ----- INST_TAG
--I_TX_CSUM_FIFO  : entity xps_ll_temac_v1_01_b.status_fifo(STRUCTURE)
--      port map (
--         clk                  => LLTemac_Clk,           -- in 
--         din                  => lL2CSFIFO_data,        -- in
--         rd_en                => iP2CSFIFO_RdReq,       -- in
--         rst                  => LLTemac_Rst,           -- in 
--         wr_en                => lL2CSFIFO_wren,        -- in 
--         dout                 => cSFIFO2IP_Data,        -- out
--         empty                => cSFIFO2IP_Empty,       -- out
--         full                 => cSFIFO2LL_full,        -- out
--         overflow             => open,                  -- out
--         prog_empty           => open,                  -- out
--         prog_full            => open,                  -- out
--         valid                => open,                  -- out
--         underflow            => open,                  -- out
--         wr_ack               => open                  -- out
--         );
-- INST_TAG_END ------ End INSTANTIATION Template ------------
 
-- The following code must appear in the VHDL architecture
-- body. Substitute your own instance name and net names.

------------- Begin Cut here for INSTANTIATION Template ----- INST_TAG
--I_TX_FIFO : entity xps_ll_temac_v1_01_b.tx_rx_fifo(STRUCTURE)
--      port map (
--         clk                  => LLTemac_Clk,           -- in
--         din                  => lL2TXFIFO_Data,        -- in
--         rd_en                => ip2TXFIFO_RdReq,       -- in 
--         rst                  => LLTemac_Rst,           -- in
--         wr_en                => lL2TXFIFO_Wren,        -- in 
--         dout                 => txFIFO2IP_Data,        -- out
--         empty                => txFIFO2IP_Empty,       -- out
--         full                 => txFIFO2LL_Full,        -- out
--         overflow             => open,
--         prog_empty           => open,
--         prog_full            => open,
--         valid                => txFIFO2IP_RdAck,
--        underflow            => open,
--         wr_ack               => open
--         );
-- INST_TAG_END ------ End INSTANTIATION Template ------------



 I_TX_FIFO : sync_fifo_v5_0
    generic map(
              c_dcount_width       => log2(C_TEMAC_TXFIFO/4),
              c_enable_rlocs       => 0,
              c_has_dcount         => 1,
              c_has_rd_ack         => 0,
              c_has_rd_err         => 0,
              c_has_wr_ack         => 0,
              c_has_wr_err         => 0,
              c_memory_type        => 1,
              c_ports_differ       => 0,
              c_rd_ack_low         => 0,
              c_read_data_width    => 36,
              c_read_depth         => (C_TEMAC_TXFIFO/4),
              c_rd_err_low         => 0,
              c_wr_ack_low         => 0,
              c_wr_err_low         => 0,
              c_write_data_width   => 36,
              c_write_depth        => (C_TEMAC_TXFIFO/4)
         )
    port map(
              CLK          => LLTemac_Clk,           -- in
              SINIT        => LLTemac_Rst,           -- in
              DIN          => lL2TXFIFO_Data,        -- in
              WR_EN        => lL2TXFIFO_Wren,        -- in
              RD_EN        => ip2TXFIFO_RdReq,       -- in
              DOUT         => txFIFO2IP_Data,        -- out
              FULL         => txFIFO2LL_Full,        -- out
              EMPTY        => txFIFO2IP_Empty,       -- out
              RD_ACK       => open,                  -- out
              WR_ACK       => open,                  -- out
              RD_ERR       => open,                  -- out
              WR_ERR       => open,                  -- out
              DATA_COUNT   => sig_data_count         -- out
         );

--GAB 10/19/07 modified to fix timing path
--txFIFO2IP_aFull <= '1' when conv_integer(sig_data_count)>(C_TEMAC_TXFIFO/4)-2 else '0';
txFIFO2IP_aFull_cmb <= '1' when conv_integer(sig_data_count)>(C_TEMAC_TXFIFO/4)-3 else '0';

REG_AFULL : process(LLTemac_Clk)
    begin
        if(LLTemac_clk'EVENT and LLTemac_Clk='1')then
            if(LLTemac_Rst='1')then
                txFIFO2IP_aFull <= '0';
                csfifo2ll_full_reg <= '0';
                lL2CSFIFO_wren_dly1 <='0';
            else
                txFIFO2IP_aFull <= txFIFO2IP_aFull_cmb;
                csfifo2ll_full_reg <= csfifo2ll_full;
                lL2CSFIFO_wren_dly1 <= ll2csfifo_wren;
            end if;
        end if;
    end process REG_AFULL;
    

--
-- Start Async FIFO 6.1 model --------------------------------------                                                   
--
-- I_TX_FIFO : async_fifo_v5_1 
--    generic map(
--         C_DATA_WIDTH       =>  36,
--         C_ENABLE_RLOCS     =>  0,
--         C_FIFO_DEPTH       =>  (C_TEMAC_TXFIFO/4),
--         C_HAS_ALMOST_EMPTY =>  0,
--         C_HAS_ALMOST_FULL  =>  1,
--         C_HAS_RD_ACK       =>  0,
--         C_HAS_RD_COUNT     =>  0,
--         C_HAS_RD_ERR       =>  0,
--         C_HAS_WR_ACK       =>  0,
--         C_HAS_WR_COUNT     =>  0,
--         C_HAS_WR_ERR       =>  0,
--         C_RD_ACK_LOW       =>  0,
--         C_RD_COUNT_WIDTH   =>  14,
--         C_RD_ERR_LOW       =>  0,
--         C_USE_BLOCKMEM     =>  1,
--         C_WR_ACK_LOW       =>  0,
--         C_WR_COUNT_WIDTH   =>  14,
--         C_WR_ERR_LOW       =>  0   
--         )
--    port map(
--         Din            => lL2TXFIFO_Data,        -- in 
--         Wr_en          => lL2TXFIFO_Wren,        -- in 
--         Wr_clk         => LLTemac_Clk,           -- in 
--         Rd_en          => ip2TXFIFO_RdReq,       -- in 
--         Rd_clk         => LLTemac_Clk,            -- in 
--         Ainit          => LLTemac_Rst,           -- in 
--         Dout           => txFIFO2IP_Data,        -- out
--         Full           => txFIFO2LL_Full,        -- out
--         Empty          => txFIFO2IP_Empty,       -- out
--         Almost_full    => txFIFO2IP_aFull,       -- out
--         Almost_empty   => open,                  -- out
--         Wr_count       => open,                  -- out                           
--         Rd_count       => open,                  -- out                            
--         Rd_ack         => open,                  -- out 
--         Rd_err         => open,                  -- out 
--         Wr_ack         => open,                  -- out 
--         Wr_err         => open                   -- out 
--         ); 

I_TX_TEMAC_IF : entity xps_ll_temac_v1_01_b.tx_temac_if 
   generic map(
      C_RESET_ACTIVE          => C_RESET_ACTIVE,
      C_CLIENT_DWIDTH         => C_CLIENT_DWIDTH
      )

   port map(
      LLTemac_Clk             => LLTemac_Clk,            -- in  
      LLTemac_Rst             => LLTemac_Rst,            -- in  
                             
      IP2TXFIFO_RdReq         => ip2TXFIFO_RdReq,        -- out 
      TXFIFO2IP_Data          => txFIFO2IP_Data,         -- in  
      TXFIFO2IP_RdAck         => txFIFO2IP_RdAck,        -- in  
      TXFIFO2IP_Empty         => txFIFO2IP_Empty,        -- in  
      TXFIFO2IP_Und           => txFIFO2IP_Und,          -- in  
      TXFIFO2IP_Ovr           => txFIFO2IP_Ovr,          -- in  
                              
      TXFIFO_Und_Intr         => TXFIFO_Und_Intr,        -- out 
                              
      CSFIFO2IP_Ovr           => cSFIFO2IP_Ovr,          -- in  
      CSFIFO2IP_Und           => cSFIFO2IP_Und,          -- in  
      CSFIFO2IP_Empty         => cSFIFO2IP_Empty,        -- in  
      CSFIFO2IP_Data          => cSFIFO2IP_Data,         -- in  
      IP2CSFIFO_RdReq         => iP2CSFIFO_RdReq,        -- out 
                             
      Tx2ClientPauseReq       => Tx2ClientPauseReq,      -- in  
      ClientEmacPauseReq      => clientEmacPauseReq_i,     -- out 
                              
      Tx_cmplt                => tx_cmplt_i,               -- out 
      Tx_Cl_Clk               => Tx_Cl_Clk,              -- in  
      ClientEmacTxd           => clientEmacTxd_i,          -- out 
      ClientEmacTxdVld        => clientEmacTxdVld_i,       -- out 
      ClientEmacTxdVldMsw     => clientEmacTxdVldMsw_i,    -- out 
      ClientEmacTxFirstByte   => clientEmacTxFirstByte_i,  -- out 
      ClientEmacTxUnderRun    => clientEmacTxUnderRun_i,   -- out 
      EmacClientTxAck         => EmacClientTxAck,        -- in  
      EmacClientTxCollision   => EmacClientTxCollision,  -- in  
      EmacClientTxRetransmit  => EmacClientTxRetransmit, -- in  
      Tx2ClientUnderRunIntrpt => Tx2ClientUnderRunIntrpt -- out 
      );
      
------------------------------------------------------------------------------
-- This process throttles the locallink while the csum is being calculated
------------------------------------------------------------------------------ 

CSUM_CALC_THROTTLE : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk)) then
         if(LLTemac_Rst='1' or lL2CSFIFO_wren_dly1='1') then
            csum_ready <= '1';
         elsif(LLTemac_EOP_n='0' and 
            LLTemac_SRC_RDY_n='0' and 
            lLTemac_DST_RDY_n_i='0') then
            csum_ready <= '0';
         end if;
      end if;
   end process;
      

      
      
end beh;
